
setClass(
		"mcqMetadata",
		slots = list(metadata="data.frame",samples="data.frame"),
		validity=function(object){
			cat("Checking mcqMetadata validity\n")
			if ('msrunfile' %in% colnames(object@metadata)) {
				cat ("mcqMetadata shouldn't have an msrunfile column\n")
				return(FALSE)
			}
			if ('msrun' %in% colnames(object@metadata)) {
				cat ("mcqMetadata shouldn't have an msrun column\n")
				return(FALSE)
			}
			cat("mcqMetadata object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcqData",
		slots = list(quantipeptides="data.frame",samples="list", groups="list",
				proteins="list",peptidesmh="data.frame", observedin="data.frame",quantify="matrix"),
		validity=function(object){
			cat("Validity\n")
			return(TRUE)
		}
)


setClass(
		"mcq.protlist",
		#very simple structure : proteins is a list with named elements
		# the names are the protein accessions
		# the elements are the protein description
		slots = list(proteins="character"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcqProtList object is valid\n")
			return(TRUE)
		}
)
setClass(
		"mcq.protlist.padjust",
		slots = list(proteins="character",pvalue="data.frame",padjust="data.frame"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.protlist.padjust object is valid\n")
			return(TRUE)
		}
)

priv.check.protpep = function(object){
	cat("Checking validity\n")
	unique_peptide <- unique(object@proteins$peptide)
	for (peptide in unique(object@peptides$peptide)) {
		if (peptide %in% unique_peptide) {

		}
		else {
			cat(paste(class(object), " ERROR : peptide ",peptide," is not described in proteins\n", sep=""))
			return(FALSE)
		}
	}
	cat(paste(class(object)," object is valid\n", sep=""))
	return(TRUE)
}

setClass(
		"mcqProtPep",
		slots = list(proteins="data.frame",peptides="data.frame", metadata="mcqMetadata", nona="data.frame"),
		validity=function(object){
			return(priv.check.protpep(object))
		}
)

setClass(
		"mcq.skyline",
		slots = list(proteins="data.frame",peptides="data.frame", metadata="mcqMetadata", nona="data.frame"),
		validity=function(object){
			return(priv.check.protpep(object))
		}
)



setClass(
		"mcqProtPCA",
		slots = list(proteins="data.frame",metadata="mcqMetadata",samples="data.frame",pca="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcqProtPCA object is valid\n")
			return(TRUE)
		}
)

setClass(
		"mcqPeptidePCA",
		slots = list(peptides="data.frame", proteins="data.frame",metadata="mcqMetadata",samples="data.frame",pca="list"),
		validity=function(object){
			cat("Checking validity\n")
			cat("mcqPeptidePCA object is valid\n")
			return(TRUE)
		}
)


priv.check.protq = function(object){
	cat("Checking validity\n")
	if("protein" %in% colnames(object@proteins)) {
		cat(paste(class(object), "@proteins should not have a protein column\n", sep=""))
		return(FALSE)
	}
	if("descr" %in% colnames(object@proteins)) {
	}
	else {
		cat(paste(class(object), "@proteins should have a descr column\n", sep=""))
		return(FALSE)
	}
	if("accession" %in% colnames(object@proteins)) {
	}
	else {
		cat(paste(class(object), "@proteins should have an accession column\n", sep=""))
		return(FALSE)
	}
	for (protein in object@protval$protein) {
		if (protein %in% rownames(object@proteins)) {

		}
		else {
			cat(paste(class(object), " ERROR in protval : protein '",protein,"' is not described\n", sep=""))
			return(FALSE)
		}
	}
	cat(paste(class(object)," object is valid\n", sep=""))
	return(TRUE)
}

setClass(
		"mcq.protquantity",
		slots = list(proteins="data.frame",metadata="mcqMetadata",protval="data.frame",samples="data.frame"),
		validity=function(object){
			return(priv.check.protq(object))
		}
)

setClass(
		"mcq.spectralcounting",
		slots = list(proteins="data.frame",metadata="mcqMetadata",protval="data.frame",samples="data.frame"),
		validity=function(object){
			return(priv.check.protq(object))
		}
)

setClass(
		"mcq.peakcounting",
		slots = list(proteins="data.frame",metadata="mcqMetadata",protval="data.frame",samples="data.frame"),
		validity=function(object){
			return(priv.check.protq(object))
		}
)

setClass(
		"mcq.protquantity.anova",
		slots = list(aov="list",proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultanov="data.frame", resulttukey="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.prot.anova object is valid\n")
			return(TRUE)
		}
)

setClass(
		"mcq.spectralcounting.anova",
		slots = list(aov="list",proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultanov="data.frame", resulttukey="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.spectralcounting.anova object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcq.peakcounting.anova",
		slots = list(aov="list",proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultanov="data.frame", resulttukey="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.peakcounting.anova object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcq.spectralcounting.kruskal",
		slots = list(proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultanov="data.frame", resulttukey="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.spectralcounting.kruskal object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcq.peakcounting.kruskal",
		slots = list(proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultanov="data.frame", resulttukey="list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.peakcounting.kruskal object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcq.pairwise.t.test",
		slots = list(proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", restest = "list"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.pairwise.t.test object is valid\n")
			return(TRUE)
		}
)


setClass(
		"mcq.peakcounting.glm",
		slots = list(proteins="data.frame",metadata="mcqMetadata",samples="data.frame", paramlist = "list", protval="data.frame", resultglm="data.frame"),
		validity=function(object){
			cat("Checking validity\n")

			cat("mcq.peakcounting.glm object is valid\n")
			return(TRUE)
		}
)
