
mcq.plot.pvalues <- function (object, pvalFactor, pdf = NULL, nclass=100) {
	if (priv.mcq.is.anova.object(object) || (class(object)=="mcq.pairwise.t.test")) {

	}
	else {
		stop(paste("the object ",class(object)," is not taken into account by mcq.plot.pvalue"))
	}
	if (priv.mcq.is.anova.object(object)) {
		if (pvalFactor %in% colnames(object@resultanov)) {

		}
		else {
			stop(paste("the pvalue for factor ",factorName," is not described in your anova results"))
		}
	}


	if (is.null(pdf)){
		#par(ask=TRUE)
		# a single hist : no need to pause
	}
	else{
		pdf(pdf, title=gsub('.pdf$','',pdf))
	}
	if (priv.mcq.is.anova.object(object)) {
		tpval <- object@resultanov[,pvalFactor]
		hist(tpval,main=paste("Pvalue distribution for factor \"", pvalFactor,"\"", sep=""), xlab=paste(pvalFactor," pvalues",sep=""),nclass=nclass)
	}
	else if (class(object)=="mcq.pairwise.t.test") {
		tpval <- object@restest$pair.result$pval
		hist(tpval,main=paste("Pvalue distribution for factor \"", object@paramlist$factor,"\"", sep=""), xlab=paste(object@paramlist$factor," pvalues",sep=""),nclass=nclass)
	}
	#mtext(paste("Min. : ",summary(width_peak)[1]," 1st Qu.: ",summary(width_peak)[2]," Median : ",summary(width_peak)[3]," 3rd Qu. : ",summary(width_peak)[5]," Max. : ",summary(width_peak)[6],sep=""),side=3)
	if (!is.null(pdf)){
		dev.off()
	}

}


privMcqSpectralcountAnova = function (object, factorList, kruskal=F) {
	if (is.null(object@metadata)) {
		stop("there is no metadata in your object")
	}

	if (is.vector(factorList)) {

	}
	else {
		factorList <- c(factorList);
	}
	if (kruskal) {
		if (length(factorList) > 1) {
			stop(paste("Kruskal test is only available for one factor"))
		}
		cat("performing Kruskal test\n")
	}
	object@paramlist$kruskal <- kruskal
	object@paramlist$factorList <- factorList
	object@paramlist$interactionList <- c()
	if (length(factorList) == 2) {
		object@paramlist$interactionList <- c(paste(factorList, collapse="__"))
	}
	if (length(factorList) == 3) {
		object@paramlist$interactionList <- c(paste(factorList[c(1,2)], collapse="__"),
				paste(factorList[c(1,3)], collapse="__"),
				paste(factorList[c(2,3)], collapse="__"),
				paste(factorList, collapse="__"))
	}
	if (length(factorList) > 3) {
		stop(paste("too much factors in your factor list"))
	}

	for (factorName in factorList) {
		if (factorName %in% colnames(object@metadata@metadata)) {

		}
		else {
			stop(paste("the factor ",factorName," is not described in your metadata"))
		}
	}

	# Anova.
# attention: dans certains cas l'anova est faussée par le fait que certains comptages sont constants: par exemple si un peptide est absent dans toutes les reps d'un traitement, le peak counting est à 0 dans tous ses échantillons, donc variation intra nulle, ce qui n'est pas bon pour l'ANOVA. Elle risque de ce fait de produire des "faux positifs". C'est pourquoi il faut assu se donner un seuil de différence minimale.

	options(contrasts=c("contr.sum","contr.sum"))
	tabLevels <- levels(object@count$protein)
	nbprot<- length(tabLevels)
	ni <- object@count
	ni <- merge (ni, object@metadata@metadata, by.x="msrun", by.y="row.names")
	object@resultanov<- data.frame()
	matrixAnov <- NULL
	for (ip in 1:nbprot){
		myprot <- tabLevels[ip]
		tabcour <- ni[ni$protein==myprot,]
		if (kruskal) {
			ktest <- kruskal.test(as.formula(paste("count~",paste(factorList, collapse='*'),sep="")), data = tabcour)
			resultByFactor <- c(ktest$p.value)
			names(resultByFactor) <- factorList[1]
#print (ktest$p.value)
		}
		else {
			lm.prot<-lm(paste("count~",paste(factorList, collapse='*'),sep=""), data=tabcour)

			resultByFactor <- c()
			ifactor <- 0
			for (factorName in factorList) {
				ifactor <- ifactor+1
				resultByFactor[factorName]<-anova(lm.prot)["Pr(>F)"][[1]][ifactor]
			}
			if (sum(is.na(resultByFactor)) == length(factorList)) {
				for (factorName in factorList) {
					resultByFactor[factorName] <- NA
				}
				for (factorName in object@paramlist$interactionList) {
					resultByFactor[factorName] <- NA
				}
			}
			else {
				for (factorName in object@paramlist$interactionList) {
					ifactor <- ifactor+1
					resultByFactor[factorName]<-anova(lm.prot)["Pr(>F)"][[1]][ifactor]
				}
			}
		}
#		object@resultanov<-rbind(object@resultanov,cbind.data.frame(Protein=myprot,resultByFactor))
		if (is.null(matrixAnov)) {
			matrixAnov <- matrix (resultByFactor, ncol=length(resultByFactor), nrow=nbprot)
		}
		else {
			matrixAnov[ip,]<-resultByFactor
		}
		if (ip%%10==0) cat(paste(ip," proteins\n"))
	}
	object@resultanov <- data.frame(matrixAnov)
	rownames(object@resultanov) <- tabLevels
	colnames(object@resultanov) <- names (resultByFactor)

#	rownames(object@resultanov) <- object@resultanov$Protein
#	object@resultanov$Protein <- NULL
	#rename column :
#	colnames(object@resultanov) <- factorList

	return (object)

}


privMcqSelectPeakcountAnovaSignif <- function (object,pvalue, mindiff, factor=NULL) {
	if (is.null(object@resultanov)) {
		stop("there is no anova results in your object")
	}
	if (nrow(object@resultanov) == 0) {
		stop("there is no anova results in your object")
	}
	## Sélection des protéines présentant une différence minimale entre les deux traitements les plus extrèmes.
	ni <- object@count
	ni <- merge (ni, object@metadata@metadata, by.x="msrun", by.y="row.names")
	factorName <- factor
	factorList <- colnames(object@metadata@metadata)
	if (is.null(factorName)) {
		if (length(factorList) > 1) {
			stop(paste("You must choose a factor to select significantly different proteins (add the parameter factor=).\n",
							"available factors are : ", paste(factorList,collapse=", "), "\n", sep=""))
		}
		factorName <- factorList[1]
	}
	#print(factorName)
	if (factorName %in% colnames(object@metadata@metadata)) {
	}
	else {
		stop(paste("the factor ",factorName," is not described in your metadata", sep=""))
	}
	if (factorName %in% colnames(object@resultanov)) {
	}
	else {
		stop(paste("the factor ",factorName," is not described in the results of the anova", sep=""))
	}
	tabLevels <- levels(object@count$protein)
	nbprot<- length(tabLevels)

	result.diff=NULL
	for (ip in 1:nbprot){
		myprot=tabLevels[ip]
		tabcour=ni[ni$protein==myprot,]
		mmm=aggregate(tabcour$count,list(tabcour[,factorName]),FUN=mean)
		max.mean=max(mmm$x)
		min.mean=min(mmm$x)
		diff.mean=max.mean-min.mean
		result.diff=rbind(result.diff,cbind.data.frame(Protein=myprot,diff=diff.mean))
		if (ip%%10==0) cat(paste(ip," proteins\n"))
	}

	object@paramlist$df.peak.diff <- result.diff
	## attention je prends ici le critère d'écart minimum de 5 entre mini et maxi, ce qui n'est pas beaucoup.
	resultanov<-merge(object@resultanov,result.diff,by.x="row.names", by.y="Protein")
	rownames(resultanov) <- resultanov$Row.names
	resultanov$Row.names <- NULL

	object@paramlist$pvaluethreshold <- pvalue
	object@paramlist$mindiffpeak <- mindiff
	object@paramlist$factorsignif <- factorName

	signif=resultanov[,factorName]<pvalue
# ok.signif=apply(signif,1,FUN=sum)>0 & resultanov$diff>5
	ok.signif=signif & resultanov$diff>mindiff


	#metadata="mcqMetadata", count="data.frame", resultanov="data.frame", paramlist
	nbProt = length(ok.signif)
	object@resultanov$bidon <- object@resultanov[,factorName]
	object@resultanov <- object@resultanov[ok.signif,]
	object@resultanov$bidon <- NULL
	object@resultanov <- drop.levels(object@resultanov)
	newNbProt <- length(rownames(object@resultanov))

	cat (paste(newNbProt," proteins selected. ", (nbProt - newNbProt), " proteins dropped\n", sep=""))

	#"protein msrun count"
	#cleaning object slots
	object@count <- object@count[object@count$protein %in% rownames(object@resultanov),]
	object@count <- drop.levels(object@count)

	msrunlist <- unique(object@count$msrun)
	object@metadata@metadata$bidon <- rownames(object@metadata@metadata)
	object@metadata@metadata <- object@metadata@metadata[msrunlist,]
	object@metadata@metadata$bidon <- NULL
	object@metadata@metadata <- drop.levels(object@metadata@metadata)

	return (object)

}


privMcqPlotPeakcountAnova =  function (object, factor=NULL, pdf=NULL) {
	if (is.null(object@resultanov)) {
		stop("there is no anova results in your object")
	}

	#object@paramlist$pvaluethreshold
	#object@paramlist$mindiffpeak
	#object@paramlist$factorsignif

	if (is.null(object@paramlist$factorsignif)) {
		stop("mcqPlotPeakcountingAnovaSignif only works after mcqSelectPeakcountingAnovaSignif")
	}

	tabprot <- object@count
	listprot <- unique(object@count$protein)

	factorSignif <- NULL
	if (!is.null(object@paramlist$factorsignif)) {
		factorSignif <- object@paramlist$factorsignif
		cat(paste("The factor \"", factorSignif,"\" was used to select proteins significantly different\n", sep=""))
	}

	factorName <- NULL
	if (is.null(factor)) {
		if (is.null(factorSignif)) {
			stop(paste("You must choose a factor to group proteins in a boxplot (add the parameter factor=).\n",
							"available factors are : ", paste(colnames(object@metadata@metadata),collapse=", "), "\n", sep=""))
		}
		else {
			factorName <- factorSignif
		}
	}
	else {
		factorName <- factor
		if (factorName %in% colnames(object@metadata@metadata)) {
		}
		else {
			stop(paste("the factor ",factorName," is not described in your metadata\n", sep=""))
		}
	}


	#cat(paste(length(listprot)," proteins significantly different between extrem levels of factor \"",factorSignif,  "\"\n", sep=""))

	if (is.null(pdf)){
		par(ask=TRUE)
	}
	else{
		pdf(pdf, title=gsub('.pdf$','',pdf))
	}

	tabprot <- merge(object@count, object@metadata@metadata, by.x="msrun", by.y="row.names")

	for (protein_name in listprot) {
		tabcour=tabprot[tabprot$protein==protein_name,]
		if (is.null(factorSignif)) {
			title <- paste(protein_name, " (", factorName, ")", sep="")
		}
		else {
			title <- paste(protein_name, " (", factorName, "/", factorSignif, ")", sep="")
		}
		visuprot(protein_name,tabcour,tabcour$count,tabcour[,factorName], title=title)
	}



	if (!is.null(pdf)){
		dev.off()
	}
}

