

mcq.get.dataframe <- function (object, comp=F) {
	if ((class(object) == "mcq.protquantity") || (class(object) == "mcq.peakcounting") || (class(object) == "mcq.spectralcounting")) {
		#if (is.null(object@metadata@metadata)) {
		if (nrow(object@metadata@metadata)==0) {
			tprot <- object@protval
		}
		else {
			tprot <- merge(object@protval, object@metadata@metadata, by.x="msrun", by.y="row.names")
		}

		if ((class(object) == "mcq.peakcounting")|| (class(object) == "mcq.spectralcounting")) {
			#rename q to count :
			colnames(tprot)[colnames(tprot) == "q"] <- "count"
		}

		return (tprot)
	}
	else if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		#if (is.null(object@metadata@metadata)) {
		if (nrow(object@metadata@metadata)==0) {
			return (object@peptides)
		}
		else {
			tpep <- merge(object@peptides, object@metadata@metadata, by.x="msrun", by.y="row.names")
			return (tpep)
		}
	}
	else if (priv.mcq.is.anova.object(object)) {

		tprotanova <- merge(object@protval, object@metadata@metadata, by.x="msrun", by.y="row.names")
		tprotanova <- merge(tprotanova, object@proteins, by.x="protein", by.y="row.names")
		#if (is.null(object@resultanov)) {
		if (nrow(object@resultanov)==0) {
		}
		else {
			resultannov <- object@resultanov
			#rename resultannov columns (it contains pvalues, not metadata)
			colnames(resultannov) <- paste("pval_",colnames(resultannov), sep="")
			tprotanova <- merge(tprotanova,resultannov, by.x="protein", by.y="row.names")
		}
		return (tprotanova)

	} else if ((class(object) == "mcqPeptidePCA") || (class(object) == "mcqProtPCA")) {
		if(comp) {
			#write component table
			return (privGetMcqPCADudiComponent(object))
		}
		else {
			return (privGetMcqPCADudi(object))
		}
	} else if (class(object) == "mcq.protlist") {
		n <- length(object@proteins)
		df = data.frame(accession=names(object@proteins), descr=object@proteins ,
				stringsAsFactors=FALSE)
		return (df)
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.get.dataframe",sep=""))
	}
}


mcq.write.tsv <- function (object, csv, comp=F) {
	#rownames are not displayed
	if (is.null(csv)) {
		stop("Please give an output filename")
	}
	write.table(mcq.get.dataframe(object,comp),csv, quote= FALSE,  sep="\t", dec=".",row=F)
}