############## Imputation
# remplace les données manquantes des peptides en se servant de leur corrélation avec les autres peptides de la même protéine.
# Attention, si dangereux on a autorisé beaucoup de données manquantes. (en "standard" on en a autorisé 5%, donc ici le nb de données manquantes par peptide ne dépasse pas 1 ou 2.

# L'imputation nous permettra à l'étape suivante de ne pas s'occuper du problème des données manquantes quand on va reconstituer la valeur d'une protéine à partir de la somme ou de la moyenne de ses peptides.

# Après cette étape il peut encore rester quelques données manquantes : cas où tous les peptides d'une protéine sont absents dans un échantillon.

#library("VIM")

mcq.data.imputations = function (object, factorName) {
	if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		if (factorName %in% colnames(object@metadata@metadata)) {

		}
		else {
			stop(paste("the factor ",factorName," is not described in your metadata"))
		}
		if("lognorm" %in% colnames(object@peptides)) {

		}else {
			stop("mcqProtPep object is not normalized")
		}
		proteines <- levels(object@proteins$protein)

# pour ne pas imprimer au terminal les messages de warning de irmi
		#zz=file("messages-warning-irmi.out.txt", open = "wt")
		#sink(zz)
		#sink(zz, type = "message")

		tabok <- object@peptides
		tabok <- merge(tabok, object@metadata@metadata, by.x="msrun", by.y="row.names")
		tabok$caract<-paste(tabok[,factorName],tabok$msrun,sep="--")
		tabok.nona=NULL
		for (i in 1:length(proteines)){
			tryCatch({
				getpep <- object@proteins[object@proteins$protein == proteines[i],"peptide"]
				print(i)
				sub=tabok[tabok$peptide %in% getpep,]
	# il faut le format dataframe pour que stack marche dans tous les cas après.
				toto=as.data.frame(tapply(sub$lognorm,list(sub$caract,as.character(sub$peptiz)),FUN=mean,na.rm=T))
				ech=rep(rownames(toto),length(colnames(toto)))
				if (sum(is.na(toto))>0) {
					toto=irmi(toto)
				}
				toto=cbind(ech,stack(toto))
				tabok.nona=rbind(tabok.nona,toto)
			}, error=function(e){cat("ERROR FOR protein ", proteines[i], " :",conditionMessage(e), "\n")})
 # if (i%%10==0) print(i) # sert à rien tant que sink en action
		}

# fin de redirection des messages de warning
		#sink(type = "message")
		#sink()

#reconstitution des variables de tabok.nona


		#tabok.nona$extraction=substr(tabok.nona$ech,1,4)
		tabok.nona$extraction <- substr(tabok.nona$ech,1,regexpr("--", tabok.nona$ech)-1)
		#tabok.nona$msrun=substr(tabok.nona$ech,6,nchar(as.character(tabok.nona$ech)))
		tabok.nona$msrun <- substr(tabok.nona$ech,regexpr("--", tabok.nona$ech)+2,nchar(as.character(tabok.nona$ech)))

		#tabok.nona=tabok.nona[,-which(colnames(tabok.nona)=="ech")]
		tabok.nona$ech <- NULL
		colnames(tabok.nona)[1:2]=c("lognorm","peptiz")


		#protein.data=unique(tabok[,c("peptiz","protein","descr")])
		protein.data <- merge(unique(object@proteins[,c("peptide","protein","accession","protein_description")]),unique(object@peptides[,c("peptide","peptiz")]), by="peptide" )

		tabok.nona=merge(tabok.nona,protein.data,by="peptiz")

# comptage des protéines pour lesquelles il reste une donnée manquante car tous ses peptides étaient absents dans l'échantillon
		compte.prot=table(tabok.nona$protein,tabok.nona$msrun)
		bad=compte.prot==0
		bad.prot=apply(bad,1,sum)
		n.dmq=bad.prot[bad.prot>0]
#		cat(paste("Nb de protéines totalement absentes dans au moins un échantillon:", length(n.dmq),"\n"))
		cat(paste("Number of proteins totally missing in at least one sample :" , length(n.dmq),"\n"))

		#save(tabok.nona,file="tabok.nona.RData")
		#tabok=tabok.nona
		#rm(tabok.nona)

		#print(tabok.nona[1:10,])

		object@nona <- tabok.nona

		return (object)

	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcqDataImputations",sep=""))
	}
}


