
mcq.protlist <- function(descriptions) {
#build a protein list from a string vector
	descriptions <- as.character(unique (descriptions))

	# parsing accession number
	descriptions <- str_trim(descriptions)
	accessions <- word(descriptions)

	names(descriptions) <- as.character(accessions)
	newObject <- new ("mcq.protlist", proteins = descriptions)
	return (newObject)
}

mcq.get.protlist <- function(object, fdr=NULL, pair=NULL) {
	if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		df <- unique(object@proteins[,c("accession","protein_description")])
		proteinvector <- as.character(df$protein_description)
		names(proteinvector) <- as.character(df$accession)

		newObject <- new ("mcq.protlist", proteins = proteinvector)
		return (newObject)
	} else if (priv.mcq.is.anova.object(object) || priv.mcq.is.protquant.object(object) || (class(object) == "mcq.peakcounting.kruskal")) {
		proteinvector <- as.character(object@proteins$descr)
		names(proteinvector) <- as.character(object@proteins$accession)

		newObject <- new ("mcq.protlist", proteins = proteinvector)
		return (newObject)
	} else if (class(object) == "mcq.pairwise.t.test") {
		if (!is.null(pair)) {
			if (is.vector(pair) && (length(pair) == 2)) {
				pairstr <- paste(pair, collapse="--")
				#warning(pairstr)
				tabprot <- object@restest$pair.result[object@restest$pair.result$pairs == pairstr,]
				if (nrow(tabprot) == 0) {
					pairstr <- paste(pair(1), pair(0), collapse="--")
					tabprot <- object@restest$pair.result[object@restest$pair.result$pairs == pairstr,]
				}
				if (nrow(tabprot) == 0) {
					stop(paste("There is no pair defined for the 2 sample names", paste(pair,collapse=" "), ": please check your sample names.",sep=" "))
				}
			}
			else {
				stop(paste("the pair argument must be vector containing 2 sample names",sep=""))
			}
		}
		else {
			tabprot <- object@restest$pair.result
		}
		#object@pair.result$proteins padjust
		tabprot <- merge (tabprot, object@proteins, by.x="proteins", by.y="row.names")

		if (is.null(fdr)) {
		} else {
			tabprot <- tabprot[tabprot$padjust <= fdr ,]
		}
		tabprot <- unique(tabprot[,c("accession", "descr")])
		proteinvector <- as.character(tabprot$descr)
		names(proteinvector) <- as.character(tabprot$accession)

		newObject <- new ("mcq.protlist", proteins = proteinvector)
		return (newObject)
	} else if (class(object) == "character") {
		proteinvector <- object
		names(proteinvector) <- object

		newObject <- new ("mcq.protlist", proteins = proteinvector)
		return (newObject)
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.get.protlist",sep=""))
	}
}

mcq.grep.protlist <- function(object, pattern, accession=T) {
	if (class(object) == "mcq.protlist") {
		# grep pattern on accession list
		if (accession) {
			object@proteins <- object@proteins[grepl(pattern, names(object@proteins))]
		}
		else {
			object@proteins <- object@proteins[grepl(pattern, object@proteins)]
		}
		return (object)
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.grep.protlist",sep=""))
	}
}

mcq.drop.protlist <- function(object, protlist) {
	if (class(object) == "mcq.protlist") {
		object@proteins <- object@proteins[!names(object@proteins) %in% names(protlist@proteins)]
		return (object)
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.drop.protlist",sep=""))
	}
}

mcq.select.protlist <- function(object, protlist) {
	if (class(object) == "mcq.protlist") {
		object@proteins <- object@proteins[names(object@proteins) %in% names(protlist@proteins)]
		return (object)
	}
	else if (priv.mcq.is.protquant.object(object)) {
		#metadata = object@metadata	, proteins = prot.prot, protval = tprot, samples = tsamples

		object@proteins <- object@proteins[object@proteins$accession %in% names(protlist@proteins) ,]
		object@proteins <- drop.levels(object@proteins)

		object@protval <- object@protval[object@protval$protein %in% rownames(object@proteins) ,]
		object@protval <- drop.levels(object@protval)


		object@samples$bidonnage <- rownames(object@samples)
		object@samples <- object@samples[unique(object@protval$msrun) ,]
		object@samples$bidonnage <- NULL
		object@samples <- drop.levels(object@samples)


		object@metadata@metadata$bidonnage <- rownames(object@metadata@metadata)
		object@metadata@metadata <- object@metadata@metadata[rownames(object@samples) ,]
		object@metadata@metadata$bidonnage <- NULL
		object@metadata@metadata <- drop.levels(object@metadata@metadata)


		return (object)
	}
	else if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		nbpeptides <- length(unique(object@peptides$peptide))
		nbprot <- length(unique(object@proteins$protein))

		object@proteins <- object@proteins[object@proteins$accession %in% names(protlist@proteins) ,]
		object@proteins <- drop.levels(object@proteins)

		object@peptides <- object@peptides[object@peptides$peptide %in% object@proteins$peptide,]
		object@peptides <- drop.levels(object@peptides)


		if (nrow(object@metadata@metadata) > 0){

			cat (paste("cleaning metadata\n", sep=""))
			object@metadata@metadata$bidonnage <- rownames(object@metadata@metadata)
			object@metadata@metadata <- object@metadata@metadata[unique(object@peptides$msrun),]
			object@metadata@metadata$bidonnage <- NULL
			object@metadata@metadata <- drop.levels(object@metadata@metadata)

			cat (paste("cleaning metadata samples\n", sep=""))
			object@metadata@samples$bidon <- rownames(object@metadata@samples)
			object@metadata@samples <- object@metadata@samples[rownames(object@metadata@metadata),]
			object@metadata@samples$bidon <- NULL
			object@metadata@samples <- drop.levels(object@metadata@samples)
		}

		nbpeptidesafter <- length(unique(object@peptides$peptide))
		nbprotafter <- length(unique(object@proteins$protein))
		cat (paste((nbpeptides - nbpeptidesafter), " peptides dropped\n", sep=""))
		cat (paste((nbprot - nbprotafter), " proteins dropped\n", sep=""))

		cat (paste( length(unique(object@peptides$peptiz)), " peptiz left\n", sep=""))
		cat (paste((nbpeptidesafter), " peptides left\n", sep=""))
		cat (paste((nbprotafter), " proteins left\n", sep=""))
		return (object)
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.select.protlist",sep=""))
	}
}

mcq.select.protlist.signif <- function(object, seuil_alpha=NULL, method="BH",fdr=0.01) {
	if (priv.mcq.is.anova.object(object)) {
		return (priv.mcq.select.protlist.signif(object,seuil_alpha, method))
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.plot.anova",sep=""))
	}
}

priv.mcq.select.protlist.signif <- function(object,seuil_alpha, method,fdr) {
	cat(paste("select protlist signif with method ",method, " in development\n",sep=""))
	cat(paste("length resultanov = ",length(object@resultanov),"\n",sep=""))
	cat(paste("length paramlist$factorList = ",length(object@paramlist$factorList),"\n",sep=""))
	if (is.null(seuil_alpha)) {
		cat('seuil alpha is null \n')
		protlist <- mcq.get.protlist(object)
		str(protlist)
	}else{
		cat(paste('seuil alpha is ', seuil_alpha,"\n",sep=""))
		#pval.adjusted.fdr <- p.adjust(object@resultanov,method=method)
		#ok.signif <- pval.adjusted.fdr<fdr
	#	protlist <- mcq.select.protlist(object)
	}
	newObject <- new ("mcq.protlist.padjust", proteins = protlist@proteins, pvalue = data.frame(), padjust= data.frame())
	newObject@pvalue <- object@resultanov
	# anova 1 factor
	factorName <- object@paramlist$factorList
	padjust = p.adjust(object@resultanov[,factorName],method=method)
	newObject@padjust = data.frame(padjust)
	colnames(newObject@padjust) = factorName
	#when anova 2 factors ok
	#cat('boucle\n')
	#for (factorName in 1:length(object@paramlist$factorList)) {
	#	cat(paste(object@paramlist$factorList,"\n",sep=""))
	#str(object@resultanov)
	#newObject@padjust[,factorName] <- p.adjust(object@resultanov[,factorName],method=method)
	#}
	return (newObject)
}
#
### Generic defined
#length.mcq.protlist <- function(obj) {
#	return(nchar(obj@proteins))
#}
#
#setMethod('length', "mcq.protlist",
#		function(object, ...) {
#			return(length(object@proteins));
#		}
#)