

read.mcq.skyline = function(skylineFilename) {
#	Peptide Sequence	Protein Name	Replicate Name	Precursor Mz	Precursor Charge	Product Mz	Product Charge	Fragment Ion	Retention Time	Area	Probability	QValue
#	LGEHNIDVLEGNEQFINAAK	sp|P00761|TRYP_PIG	1A	1106.055657	2	1106.055657	2	precursor	76.08	62850696	1	4.917146E-05
#	LGEHNIDVLEGNEQFINAAK	sp|P00761|TRYP_PIG	1B	1106.055657	2	1106.055657	2	precursor	75.96	79985064	1	4.9154543E-05
#	LGEHNIDVLEGNEQFINAAK	sp|P00761|TRYP_PIG	1C	1106.055657	2	1106.055657	2	precursor	76.07	52742156	1	0
	skyline  <- read.table(skylineFilename, header=TRUE, sep="\t", dec=".", na.strings=c("NA", "#N/A"), allowEscapes = FALSE, quote="")

	#rename columns
	colnames(skyline)[colnames(skyline) == "Peptide.Sequence"] <- "sequence"
	colnames(skyline)[colnames(skyline) == "Protein.Name"] <- "accession"
	colnames(skyline)[colnames(skyline) == "Replicate.Name"] <- "msrun"
	colnames(skyline)[colnames(skyline) == "Precursor.Mz"] <- "mz"
	colnames(skyline)[colnames(skyline) == "Precursor.Charge"] <- "z"
	colnames(skyline)[colnames(skyline) == "Retention.Time"] <- "rt"
	colnames(skyline)[colnames(skyline) == "Area"] <- "area"

	#keep only monoisotopic peptide :
	skyline <- skyline[skyline$Fragment.Ion == "precursor",]

	#drop NA values :
	skyline <- skyline[!is.na(skyline$area),]
	#drop 0 values :
	skyline <- skyline[(skyline$area != 0),]
	skyline <- drop.levels(skyline)


	skyline$rt=(skyline$rt * 60)

	#print (summary(skyline))
	mhplus <- (1.00727646677 * skyline$z)
	skyline$mh=((skyline$mz * skyline$z) - mhplus)
	skyline$peptide=as.factor(paste(skyline$sequence, sprintf("%04.4f",skyline$mh),sep='-'))

	proteins <- unique(skyline[,c("peptide", "accession")])
	proteins$protein <- proteins$accession
	proteins$protein_description <- proteins$accession

	peptides <- skyline

	#print (summary(skyline))
	#print (summary(proteins))
	peptides$msrunfile <- peptides$msrun


	peptide_col_names = c("msrun", "msrunfile","mz" ,"rt"   ,   "area"    ,
			  "peptide"   ,     "sequence"  ,   "z"         )
	for (i in peptide_col_names) {
		if(i %in% colnames(peptides))
		{
		}
		else {
			stop("ERROR : peptides dataframe does not contain expected columns\n")
			return (NULL)
		}
	}
	protein_col_names = c( "peptide"     , "accession"     ,        "protein",             "protein_description" )
	for (i in protein_col_names) {
		{
			if(i %in% colnames(proteins))
			{
			}
			else {
				stop("ERROR : proteins dataframe does not contain expected columns\n")
				return (NULL)
			}
		}
	}
	peptides$peptiz=as.factor(paste(peptides$peptide,peptides$z,sep='-'))
	peptides$logarea=log10(peptides$area)

	mcqProtPep <- new ("mcq.skyline", peptides = peptides	, proteins = proteins)
	rm (skyline)
	rm (peptides)
	rm (proteins)
	return(mcqProtPep)
}

mcq.select.monodetected.peptiz <- function(object) {
	if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		#select only peptiz quantified one time in one sample
		peptizgroup=as.factor(paste(object@peptides$peptiz,object@peptides$msrun,sep='__'))
		ni = table(peptizgroup)
#print (length(ni))
		peptizgroupok <- names(ni[ni == 1])
#print (length(peptizgroupok))
#return (ni)
#	print (head(peptizgroupok))
		f <- function(s) strsplit(s, "__")[[1]][1]

		peptizok <- sapply(peptizgroupok, f)

#	print (head(peptizok))
		nbpeptides <- length(unique(object@peptides$peptide))
		object@peptides <- object@peptides[object@peptides$peptiz %in% peptizok,]
		object@peptides <- drop.levels(object@peptides)
		nbpeptidesafter <- length(unique(object@peptides$peptide))
		cat (paste((nbpeptides - nbpeptidesafter), " peptides dropped\n", sep=""))
		nbprot <- length(unique(object@proteins$protein))
		object@proteins <- object@proteins[object@proteins$peptide %in% object@peptides$peptide,]
		object@proteins <- drop.levels(object@proteins)
		nbprotafter <- length(unique(object@proteins$protein))
		cat (paste((nbprot - nbprotafter), " proteins dropped\n", sep=""))

		return (object)

		#number_of_xic=aggregate(object@peptiz,list(peptidgroup),FUN=sd)

		#peptizok <-
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.select.monodetected.peptiz",sep=""))
	}

}
