
#########################################################################################################################################################
########################################################## Installation du paquet MassChroqR ############################################################
#########################################################################################################################################################

# voir le fichier "lisezmoi.txt"

# chargement du paquet
	library('MassChroqR')

# changement du répertoire de travail (remplacer /chemin/vers/repertoire_de_travail par votre arborescence)
	setwd ('/chemin/vers/repertoire_de_travail')
	setwd ('/tmp')
	


############################################################################################################################################################################
################################# Mise en forme des données  ###############################################################################################################
############################################################################################################################################################################
### NB : le fichier XIC_result_prot.tsv ne doit pas contenir d'apostrophes sous peine de lecture incomplète !!

################################# Création de l'objet protPep

# chargement des données d'identification et de quantification sorties de MassChroQ: mettre le fichier prot en premier
### NB: les aires sous le pic sont transformées en log (base 10) à cette étape
	protPep <- read.mcqProtPep('/gorgone/pappso/formation/Decembre2014/XIC_result_prot.tsv', '/gorgone/pappso/formation/Decembre2014/XIC_result_pep.tsv')

# visualiser la structure de l'objet protPep
	str(protPep)

# affichage du résumé du contenu de l'objet protPep
	summary (protPep)
#number of proteins : 
#number of XICs : 
#number of peptiz : (=  peptide-charge)
#number of peptides : 
#number of samples : 
	

################################# Création des métadata contenant la description des échantillons

# export d'un template dans lequel pourront être ajouté les métadata
	mcq.write.metadata.tsv(protPep, 'metadata.csv')

# Ajout des metadata: ouvrir le fichier outfile.csv sous libreoffice ou excel, ajouter des colonnes (par exemple:traitement, rep , génotype, etc. ) permettant de décrire les échantillons et enregistrer en conservant le format csv

# import des métadata
	metadata <- read.mcq.metadata.ods('/gorgone/pappso/formation/Decembre2014/metadata.ods')

# Ajout des métadata à l'objet protPep
	protPep <- mcq.merge.metadata(protPep, metadata)


# Si nécessaire sélection du jeu de données par suppression des échantillons non souhaités :
	#protPep <- mcq.drop(protPep, 'msrun', c('samp0','samp1','samp10','samp11','samp12','samp13'))

############################################################################################################################################################################
################################# Vérification de la qualité des données de chromatographie ################################################################################
############################################################################################################################################################################
### NB: on vérifie la largeur moyenne des pics et la déviation des temps de rétention pour éviter les missmatch et les erreurs eventuelles d'attribution de masse à un pic

# affichage de l'histogramme de distribution de la largeur moyenne des pics
	mcq.plot.peak.width(protPep)					# affichage à l'écran
	mcq.plot.peak.width(protPep, pdf='peak.pdf') 	# pour enregistrer le graphique sous format pdf

# suppression des XICs dont la largeur est trop longue (temps de rétention en secondes. ici, 100 sec)
	protPep <- mcq.drop.peak.width(protPep, cutoff=100)

# affichage du résumé du contenu de l'objet protPep après suppression des peptides
	summary(protPep)

# affichage de l'histogramme de distribution de la déviation des temps de rétention
	mcq.plot.retention.time.sd(protPep)							# affichage à l'écran et paramètre "limit" par défaut
	mcq.plot.retention.time.sd(protPep, pdf="RT.pdf",limit=40)  # limiter l'affichage des temps de rétention à 40s maximum (ici)

# suppression des peptides ayant une déviation supérieure à un seuil à déterminer en fonction de l'histogramme précédent (ici, 20 sec)
	protPep <- mcq.drop.retention.time.sd(protPep, cutoff=20)

# affichage du résumé du contenu de l'objet protPep après suppression des peptides
	summary(protPep)

# Statistiques exploratoires : ACP sur les peptides (peptiz / échantillons)
	peptidepca <- mcq.peptide.pca(protPep)

# Statistiques exploratoires : ACP sur les peptides (peptiz / extraction-rep, colonnes du fichiers metadata)
	peptidepca <- mcq.peptide.pca(protPep, c("extraction","rep"))

# écriture du résultat de l'ACP dans un fichier pour un traitement dans un logiciel externe
	mcq.write.tsv(peptidepca, csv="peptidepca.tsv")

# visualisation des axes de l'ACP
	mcq.plot.peptide.pca(peptidepca, colors="extraction",pdf="peptideACP.pdf")


############################################################################################################################################################################
################################# Analyse par 'Peakcounting' (Variations qualitatives) ################################################################################
############################################################################################################################################################################


# comptage du nombre de 'peak' (= peptide-charge) par protéine dans chaque échantillon
	pkcount <- mcq.peakcounting(protPep)

# mindiff => différence minimum entre min max (nombres de pics)
	pkcount.mindiff <- mcq.select.mindiff(pkcount, 5, factor='extraction')

# export en format csv (2 présentations possibles)
	mcq.write.tsv(pkcount.mindiff, csv='pkcount_mindiff5.tsv')
	mcq.write.compar.tsv(pkcount.mindiff,csv='pkcount_mindiff5_compar.tsv',sampleid=F)

# visualisation du nombre total de 'peaks', estimation de la qualité des échantillons et éventuellement enlever des échantillons
	mcq.plot.peak.per.samples(pkcount,pdf='PeakcountingPerSample.pdf')
	

#si un échantillon est "hors type", on peut le supprimer du jeu de données :
	summary(protPep)
	bis <- mcq.drop(protPep, 'msrun', 'samp11')
	summary(bis)
#dans ce cas, il faudra recompter le nombre de 'peak' sur ce nouveau jeu de données

# Statistique décisionnelle sur les données de 'Peakcounting' : 
# ANOVA 
	#si un facteur
	pkcount.anova <- mcq.anova(pkcount.mindiff,'extraction') 
	#si plusieurs facteurs
	#pkcount <- mcqPeakcountingAnova(pkcount,c('extraction','rep.bis'))

# Créer un tableau contenant le counting de peak par protéines et les résultats de l'ANOVA
	dfpeakcountinganova <- mcq.get.dataframe(pkcount.anova)
# Affichage des premières lignes du dataframe obtenu
	head(dfpeakcountinganova)

# Exporter les résultats de l'ANOVA au format csv
	mcq.write.compar.tsv(pkcount.anova, 'pkcount_anova.tsv')

# pvalue => seuil de pvalue
	pkcount.anova.signif <- mcq.select.anova.signif(pkcount.anova, pvalue=0.01,factor='extraction')

# Boxplot des protéines signif en fonction du facteur choisi
# (par défaut, c'est le facteur utilisé dans l'ANOVA qui sera pris en compte)
	mcq.plot.peakcounting.anova(pkcount.anova.signif, pdf='AnovaSignifPeakCounting.pdf')

# Créer un tableau contenant les résultats de l'anova du peak counting et les résultats de l'anova pour les protéines avec un effet traitement significatif
	dfpeakcountinganovasignif <- mcq.get.dataframe(pkcount.anova.signif)

# Exporter les résultats pour les protéines significatives au format csv
	mcq.write.tsv(pkcount.anova.signif, 'pkcount_anova_signif.csv')

# Combien de protéines avons-nous maintenant ?
	summary (protPep)

# ou
# KRUSKAL WALLIS pour les données discrètes, indépendantes
	pkcount.kruskal <- mcq.peakcounting.kruskal(pkcount.mindiff,'extraction')

# Exporter les résultats du test au format csv
	mcq.write.compar.tsv(pkcount.kruskal, 'pkcount_kruskal_mindiff5.tsv')
# Exporter les comptages et les résultats du test au format csv
	mcq.write.merge.tsv(pkcount,pkcount.kruskal,'pkcount_kruskal_mindiff5_info.tsv',sampleid=F)

# Sélectionner les protéines significatives (pvalue => seuil de pvalue)
	pkcount.kruskal.signif <- mcq.select.kruskal.signif(pkcount.kruskal, pvalue=0.05)
# Boxplot des protéines signif en fonction du facteur choisi
# (par défaut, c'est le facteur utilisé dans l'ANOVA qui sera pris en compte)
	mcq.plot.peakcounting.kruskal(pkcount.kruskal.signif, pdf='KruskalSignifPeakCounting_mindiff5.pdf')

# Récupérer la liste des protéines significatives déterminée par le KruskalWallis
	protlist.signif = mcq.get.protlist(pkcount.kruskal.signif)
# Sélectionner les comptages des protéines qui sont présentes dans la liste
	pkprot.signif = mcq.select.protlist(pkcount, protlist.signif)

# Exporter les résultats signifs (count + pvalue) au format csv
	mcq.write.merge.tsv(pkprot.signif,pkcount.kruskal.signif,'pkcount_kruskal_signif_info.tsv',sampleid=F)

############################################################################################################################################################################
################################# Analyse quantitative (XIC)   #############################################################################################################
############################################################################################################################################################################

################################# Normalisation des données (les aires sont en log10)

# Visualisation des histogrammes de distribution des intensités de peptide dans chaque échantillon
	mcq.plot.distrib.qlog(protPep) # sortie standard
	mcq.plot.distrib.qlog(protPep,pdf="DistribQlog.pdf") # fichier

# Visualisation des histogrammes de distribution des différences des log d'intensité de peptide entre un échantillon donné et l'échantillon choisi comme référence (ici samp0)
	mcq.plot.distrib.qlog.diff.ref(protPep, 'samp0', pdf='distrib_visu.pdf')

# Visualisation de la corrélation entre les intensités de peptides mesurées dans un échantillon donné et celles mesurées dans l'échantillon de référence (ici samp0)
	mcq.plot.distrib.qlog.comet.diff(protPep, 'samp0') # sortie standard
	mcq.plot.distrib.qlog.comet.diff(protPep, 'samp0','CometbyRef.pdf') # fichier

# Normalisation par la médiane des différences d'intensité de peptides
	protPep <- mcq.normalize.by.sampleref(protPep, 'samp0')
	
# Visualisation de l'effet de la normalisation
	mcq.plot.normalization.effect(protPep, pdf='normalization_effect.pdf')


################################# Filtre des données : sélection des peptides reproductibles, des peptides spécifiques et des peptides corrélés d'une même protéine

# Histogramme du nombre d'échantillons dans lesquels un peptide a été quantifié
	mcq.plot.peptide.repeatability(protPep, pdf='PeptideRepetables.pdf')

# Suppression des peptides non répétables (présents dans moins de 5% des échantillons)
# pct.max.mq => % max de valeurs manquantes
	protPep <- mcq.select.repeatable.peptides(protPep, pct.max.mq=0.05) 

#Quantification des protéines uniquement avec leurs peptides spécifiques
# Histogramme du nombre de protéines par peptide
	mcq.plot.peptides.shared.by.prot(protPep, pdf = 'PeptidesPartages.pdf')

# Suppression des peptides communs à plusieurs protéines
	protPep <- mcq.drop.shared.peptides(protPep)

# Selon nombre de modalités du facteur choisir étape a ou étape b
# a /Si le facteur "traitement" présente au moins 3 conditions (modalités) : filtre sur la correlation entre les peptides d'une même protéine
	protPep <- mcq.drop.uncorrelated.peptides(protPep, 'extraction')
# b /Si le facteur "traitement" présente moins de 3 modalités : on ne garde que les protéines quantifiées par au moins 2 peptides
	protPep <- mcq.select.2pepbyprot(protPep)

# Ecriture des intensités par peptides et par échantillons avec les valeurs manquantes
	mcq.write.compar.tsv(protPep, csv="protPepna.tsv")

################################# Imputation des valeurs manquantes par régression linéaire
	protPepnona <- mcq.data.imputations(protPep, 'extraction')

# Ecriture des intensités par peptides et par échantillons avec valeurs imputées
	mcq.write.compar.tsv(protPepnona, csv="protPepnona.tsv")

################################# Représentation ACP à partir des intensités de peptide
	pcapeptide <- mcq.peptide.pca(protPep)
# représentation des 5 premiers axes de l'ACP.
# On choisit de colorier les échantillons en fonctions du facteur traitement (colors='traitement')
	mcq.plot.peptide.pca(pcapeptide, colors='extraction', pdf='peptide_acp.pdf')
# coordonnées spatiales des échantillons
	mcq.write.tsv(pcapeptide, 'pcapeptide.csv')
# écriture des composantes (peptides/z)
	mcq.write.tsv(pcapeptide, 'pcapeptide_comp.csv', comp=T)


################################# Calcul de la quantité par protéine

# Calcul des quantités de protéine (somme des peptides)
	qprot <- mcq.protquantity(protPepnona)

# Histogramme de distribution des écart-types des quantités de protéine
	mcq.plot.protein.value.sd(qprot, pdf='SD_logQNorm.pdf')
	head(qprot@protval)

#Création d'un tableau à 2 dimensions : échantillons/protéines
	tacp <- mcq.get.compar(qprot)
	head(tacp)

# Export au format tabulé 
	mcq.write.compar.tsv(qprot, 'tacp_na.csv')

# dans les cas où il n'y avait pas de peptide de la protéine présent dans l'échantillon, on a encore un NA.
# on remplace le NA par la valeur min trouvée pour cette protéine dans l'expérience.
	qprot <- mcq.replace.na.protein.value.by.min(qprot)

	mcq.write.compar.tsv(qprot, 'tacp_nona.csv')

	mcq.plot.protein.value.sd(qprot, pdf ='SD_logQNormMIN.pdf')



# Elimination des protéines qui ne varient pas beaucoup
# cutoff => seuil minimum accepté pour l'écart type en valeur absolue (0.05=trop faible pour une valeur absolue)
# Regarder l'ecart type et adapter en fonction
	test <- mcq.drop.protquantity.sd(qprot,cutoff=0.05)
	mcq.plot.protein.value.sd(test)
	mcq.plot.protein.value.sd(qprot)

################################# Représentation ACP à partir des quantités de protéine

# AU CHOIX :
# ACP sur les protéines par échantillons
	pca <- mcq.protein.pca(qprot)
# ACP sur les protéines par extraction-rep (colonnes du fichier metdata)
	pca <- mcq.protein.pca(qprot,c('extraction','rep'))
############

# export du résultat de l'ACP dans un fichier pour manipulations dans un logiciel externe
# coordonnées spatiales des échantillons
	mcq.write.tsv(pca, 'proteinpca.csv')
# écriture des composantes (protéines)
	mcq.write.tsv(pca, 'proteinpca_comp.csv', comp=T)

#visualisation des résultats de l'ACP (écrit dans un fichier pdf)
	mcq.plot.protein.pca(pca, colors='extraction', pdf='acp.pdf')

################################# ANOVA, Heatmap et clustering sur les protéines

#ANOVA à 1 facteur
	anova <- mcq.anova (qprot, 'extraction')
# Visualisation de l'histogramme des pvalues de l'ANOVA
	mcq.plot.pvalues(anova, "extraction") 
	mcq.plot.pvalues(anova, "extraction", pdf="anova_hist_extraction.pdf") 
# 0.05 => seuil de signif sur le facteur "extraction"
	anova.signif <- mcq.select.anova.signif(anova, 0.05, factor='extraction')
# Boxplot des protéines signif
	mcq.plot.anova(anova.signif)
# Ecriture des résultats de l'ANOVA dans un fichier :
# => totalité des résultats
	mcq.write.tsv(anova, 'anova.csv')
# => uniquement les protéines signif
	mcq.write.tsv(anova.signif, 'anova_signif.csv')

# on récupère la liste des protéines significatives déterminée par l'ANOVA
	protlist.signif <- mcq.get.protlist(anova.signif)
# on sélectionne les protéines quantifiées qui sont présentes dans la liste
	qprot.signif <- mcq.select.protlist(qprot, protlist.signif)

# on peut visualiser ces protéines sur un heatmap en fonction d'un facteur
	mcq.plot.heatmap(qprot.signif, factor="extraction")
	mcq.plot.heatmap.meanrep(qprot.signif,factor="extraction",'classification_by_conditions.pdf')

# Regroupement des protéines en clusters
	mcq.plot.cluster(qprot.signif,factor="extraction",nbclust=8,pdf='cluster.pdf')
	mcq.plot.cluster.meanrep(qprot.signif,factor="extraction", nbclust=8, pdf='cluster_by_conditions.pdf')

# ecriture des infos clusters
	mcq.cluster.meanrep.write.csv(qprot.signif,factor="extraction", nbclust=8, file='infos_cluster.csv')

# Rapport des aires (dans le cas de comparaisons de deux conditions)
	mcq.plot.area.ratio(qprot.signif,factor="extraction",pdf="AreaRatio_signif_5percent.pdf")

################################## Récupération des informations de Spectral Counting ##################################################
# Pour éventuellement merger les données de Spectral Counting, Peak counting et XIC

# ATTENTION pour l'instant preparer la feuille comparspectra dans un fichier csv (bug read.ods pour fichier avec plusieurs onglets)
	spectral.count = read.mcq.spectralcounting('/gorgone/pappso/formation/Decembre2014/comparspectra.csv')

# merge avec les metadata
	spectral.count = mcq.merge.metadata(spectral.count, metadata)

# Si nécessaire sélection du jeu de données par suppression des échantillons non souhaités :
	#spectral.count <-  mcq.drop(spectral.count, 'msrun',c('samp0','samp1','samp10','samp11','samp12'))

# visualisation du nombre total de 'spectre', estimation de la qualité des échantillons et éventuellement enlever des échantillons
	
	mcq.plot.peak.per.samples(spectral.count,pdf='SpectralcountingPerSample.pdf')

# 5 => différence minimum entre min max (nombres de pics)
	spectral.count.mindiff =  mcq.select.mindiff(spectral.count, 5, 'extraction')

# export en format csv (2 présentations possibles)
	mcq.write.tsv(spectral.count.mindiff, csv='spectralcount_mindiff5.tsv')
	mcq.write.compar.tsv(spectral.count.mindiff,csv='spectralcount_mindiff5_compar.tsv',sampleid=F)

# Statistique décisionnelle sur les données de 'SpectralCounting': 
# ANOVA 
	spectral.count.anova <- mcq.spectralcounting.anova(spectral.count.mindiff,'extraction')

# Ecriture fichier resultat anova
	mcq.write.compar.tsv(spectral.count.anova, 'spectralcount_anova.csv')
 
# Ecriture fichier spectral counting count+pvalue
	mcq.write.merge.tsv(spectral.count.mindiff,spectral.count.anova,'spectralcount_anova_info.csv',sampleid=F)

# 0.01 => seuil de pvalue
	spectral.count.anova.signif <- mcq.select.anova.signif(spectral.count.anova, 0.01, factor='extraction')
# Effectuer un boxplot des protéines signif en fonction du facteur choisi
# (par défaut, c'est le facteur utilisé dans l'anova qui sera pris en compte)
	mcq.plot.proteins(spectral.count.anova.signif, pdf='AnovaSignifSpectralCounting.pdf')

# Créer un tableau contenant les résultats de l'anova du spectral counting et les résultats de l'ANOVA pour les protéines avec un effet traitement significatif
	dfspectral.count.anova.signif <- mcq.get.dataframe(spectral.count.anova.signif)

# Exporter les résultats pour les protéines significatives au format csv
	mcq.write.tsv(spectral.count.anova.signif, 'spectral_count_anova_signif.csv')

# ou
# KRUSKAL WALLIS pour les données discrètes, indépendantes
	spectral.count.kruskal <- mcq.spectralcounting.kruskal(spectral.count.mindiff,'extraction')

# Exporter les résultats du test au format csv
	mcq.write.compar.tsv(spectral.count.kruskal, 'spectralcount_kruskal_mindiff5.tsv')
# Exporter les comptages et les résultats du test au format csv
	mcq.write.merge.tsv(spectral.count.mindiff,spectral.count.kruskal,'spectralcount_kruskal_mindiff5_info.tsv',sampleid=F)

# Sélectionner les protéines significatives (pvalue => seuil de pvalue)
	spectral.count.kruskal.signif <- mcq.select.kruskal.signif(spectralcount.kruskal, pvalue=0.05)
# Boxplot des protéines signif en fonction du facteur choisi
# (par défaut, c'est le facteur utilisé dans le Kruskal qui sera pris en compte)
	mcq.plot.spectralcounting.kruskal(spectral.count.kruskal.signif, pdf='KruskalSignifPeakCounting_mindiff5.pdf')

# Récupérer la liste des protéines significatives déterminée par le KruskalWallis
	protlist.signif = mcq.get.protlist(spectral.count.kruskal.signif)
# Sélectionner les comptages des protéines qui sont présentes dans la liste
	spectralprot.signif = mcq.select.protlist(spectral.count, protlist.signif)

# Exporter les résultats signifs (count + pvalue) au format csv
	mcq.write.merge.tsv(spectralprot.signif,spectral.count.kruskal.signif,'spectralcount_kruskal_signif_info.tsv',sampleid=F)

# Visualiser ces protéines sur un heatmap en fonction d'un facteur
	mcq.plot.heatmap(spectral.count.kruskal.signif, factor="extraction",pdf='Heatmap.SCprot_signif_5percent.pdf')

# Ecriture du tableau de synthèse (en respectant l'ordre spectralcounting, peakcounting, XIC chacun accompagnés de leurs statistiques)
	mcq.write.synthesis.tsv(spectral.count,spectral.count.kruskal,pkcount,pkcount.kruskal,qprot,anova,'Synthesis_tab.tsv')

