priv.mcq.is.protquant.object = function (object) {
	if ((class(object) == "mcq.protquantity") || (class(object) == "mcq.peakcounting") || (class(object) == "mcq.spectralcounting")) {
		return (TRUE)
	}
	return (FALSE)
}

mcq.protquantity = function (object, method="sum") {
	if ((class(object) == "mcqProtPep") || (class(object) == "mcq.skyline")) {
		aggregate.function <- "sum"
		if (method == "mean") {
			aggregate.function <- "mean"
		} else if (method == "sum") {
		}
		else {
			stop(paste("unknown method ",method," for mcq.protquantity function on \"", class(object),"\" object.",sep=""))
		}
		cat(paste("compute protein quantities with the ",aggregate.function," of peptide intensities\n",sep=""))
		if("lognorm" %in% colnames(object@nona)) {
			##### construction de la valeur de la protéine par somme des peptides.
# comme les données manquantes ont été remplacées, les sommes sont faites sur les mêmes peptides dans tous les échantillons.
# Ce sont les area et non des log qui doivent être sommés.
			tprot=aggregate(10^object@nona$lognorm,list(object@nona$protein,object@nona$msrun),FUN=aggregate.function, na.rm=TRUE)

		}else {
			warning(paste("no imputations in \"", class(object),"\" object",sep=""))
			if("areanorm" %in% colnames(object@peptides)) {
				table <- merge(object@peptides[, c("peptide", "msrun","areanorm","lognorm")], object@proteins[, c("peptide", "protein")], by="peptide")
				tprot=aggregate(10^table$lognorm,list(table$protein,table$msrun),FUN=aggregate.function, na.rm=TRUE)
			} else {
				warning(paste("data are not normalized in \"", class(object),"\" object",sep=""))
				table <- merge(object@peptides[, c("peptide", "msrun","area","logarea")], object@proteins[, c("peptide", "protein")], by="peptide")
				tprot=aggregate(10^table$logarea,list(table$protein,table$msrun),FUN=aggregate.function, na.rm=TRUE)
			}
		}

		colnames(tprot)=c("protein","msrun","q")
		tprot$q=log10(tprot$q)
#réculération du descripteur
		prot.prot=unique(object@proteins[,c("protein","accession", "protein_description")])
		rownames(prot.prot) <- prot.prot$protein
		prot.prot$protein <- NULL

		tsamples = unique(object@peptides[,c("msrun","msrunfile")])
		rownames(tsamples) <- tsamples$msrun
		tsamples$msrun <- NULL


		#rename protein_description :
		colnames(prot.prot)[colnames(prot.prot) == "protein_description"] <- "descr"


		return (new ("mcq.protquantity", metadata = object@metadata	, proteins = prot.prot, protval = tprot, samples = tsamples))

	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.protquantity",sep=""))
	}
}


mcq.replace.na.protein.value.by.min = function (object) {
	if (class(object) == "mcq.protquantity") {

		tacp= as.data.frame(tapply (object@protval$q, list(object@protval$protein,object@protval$msrun), FUN=sum))

# remplacement des données manquantes restantes (tous peptides manquant dans le msrun) par le minimum de la protéine dans l'expérience.
# Attention : part d'arbitraire ici. Mais ne concerne qu'un nbre minime de protéines normalement.
# On les compte:
		cat(paste("Number of protein quantities not available :",sum(is.na(tacp)),"\n"))

		min.prot=apply(tacp,1,min,na.rm=T)
		min.prot.mat=matrix(rep(min.prot,dim(tacp)[2]),nrow=dim(tacp[1]))
		min.prot.mat=as.data.frame(min.prot.mat)
		min.prot.mat[!is.na(tacp)]=0
		tacp[is.na(tacp)]=0
		tacp=tacp+min.prot.mat



# retour à format "1 colonne"
		tacp$protein=rownames(tacp)
		object@protval <- melt(tacp,na.rm=T) # na.rm=T: les données manquantes explicites dans le tacp  deviennent implicites (en principe on n'en a plus de toutes facons)
		colnames(object@protval)[colnames(object@protval)=="variable"]="msrun"
		colnames(object@protval)[colnames(object@protval)=="value"]="q"


		return (object)

	}
	else {
		stop("input is not an mcq.protquantity object")
	}
}


mcq.plot.protein.value.sd= function (object,pdf=NULL) {
	if (class(object) == "mcq.protquantity") {

		#tacp= as.data.frame(tapply (object@protval$q, list(object@protval$protein,object@protval$msrun), FUN=sum))

		proteingroup=as.factor(object@protval$protein)
		## Suppression des peptiz dont l'écart-type du temps de rétention est trop élevé
		sdrt=aggregate(object@protval$q,list(proteingroup),FUN=sd)
		if (is.null(pdf)){
			par(ask=TRUE)
		}
		else{
			pdf(pdf, title=gsub('.pdf$','',pdf))
		}
		hist(sdrt$x,nclass=100,main="Standard deviation of protein quantities",xlab="lognorm standard deviation")
		if (!is.null(pdf)){
			dev.off()
		}
	}
	else {
		stop("input is not an mcq.protquantity object")
	}
}

mcq.drop.protquantity.sd= function (object, cutoff) {
	if (class(object) == "mcq.protquantity") {

		#tacp= as.data.frame(tapply (object@protval$q, list(object@protval$protein,object@protval$msrun), FUN=sum))

		proteingroup=as.factor(object@protval$protein)
		## Suppression des peptiz dont l'écart-type du temps de rétention est trop élevé
		sdrt=aggregate(object@protval$q,list(proteingroup),FUN=sd)

		#On enlève les quelques cas particuliers où l'écart-type est supérieur à 15 (20 ?) car correspond certainement à un mismatch
		thr_sdrt=cutoff # paramètre à changer en fonction de l'histogramme
		badprot=sdrt[!is.na(sdrt$x) & sdrt$x<thr_sdrt,] # renvoie les lignes de rt pour lesquelles sdrt>thr_sdrt.
		badprot=drop.levels(badprot$Group.1)
		cat (paste(length(badprot), " proteins dropped\n"))

		object@protval <- subset(object@protval, !protein %in% badprot)
		object@protval <- drop.levels(object@protval)
		protok <- unique(object@protval$protein)
		object@proteins <- object@proteins[protok,]
		rownames(object@proteins) <- protok

		return (object)

	}
	else {
		stop("input is not an mcq.protquantity object")
	}
}


priv.mcq.select.mindiff <-function (object, mindiff, factor=NULL) {
	## Sélection des protéines présentant une différence minimale entre les deux traitements les plus extrèmes.
	ni <- object@protval
	ni <- merge (ni, object@metadata@metadata, by.x="msrun", by.y="row.names")
	ni <- merge (ni, object@proteins, by.x="protein", by.y="row.names")
	factorName <- factor
	factorList <- colnames(object@metadata@metadata)
	if (is.null(factorName)) {
		if (length(factorList) > 1) {
			stop(paste("You must choose a factor to select significantly different proteins (add the parameter factor=).\n",
							"available factors are : ", paste(factorList,collapse=", "), "\n", sep=""))
		}
		factorName <- factorList[1]
	}
	#print(factorName)
	if (factorName %in% colnames(object@metadata@metadata)) {
	}
	else {
		stop(paste("the factor ",factorName," is not described in your metadata", sep=""))
	}

	protlist <- mcq.get.protlist(object);
	tabLevels <- names(protlist@proteins)
	nbprot<- length(tabLevels)

	result.diff <- numeric(nbprot)
	names(result.diff) <- tabLevels
	for (ip in 1:nbprot){
		myprot=tabLevels[ip]
		tabcour=ni[ni$accession==myprot,]
		mmm=aggregate(tabcour$q,list(tabcour[,factorName]),FUN=mean)
		max.mean=max(mmm$x)
		min.mean=min(mmm$x)
		result.diff[myprot]=max.mean-min.mean

		if (ip%%10==0) cat(paste(ip," proteins\n"))
	}

	result.diff <- result.diff[result.diff > mindiff]

	newNbProt <- length(result.diff)

	cat (paste(newNbProt," proteins selected. ", (nbprot - newNbProt), " proteins dropped\n", sep=""))
	#print (result.diff)

	protlist <- new ("mcq.protlist", proteins = protlist@proteins[names(result.diff)])

	#print (protlist@proteins)

	return (mcq.select.protlist(object,protlist))

}



mcq.select.mindiff <-function (object, mindiff, factor=NULL) {
	if (priv.mcq.is.protquant.object(object)) {
		return (priv.mcq.select.mindiff(object, mindiff, factor))
	}
	else {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.select.mindiff",sep=""))
	}

}



mcq.plot.protquantity = function (object, factorName, proteinList ,pdf=NULL) {
	if (!priv.mcq.is.protquant.object(object)) {
		stop(paste("the class \"", class(object),"\" is not taken into account by mcq.plot.protquantity",sep=""))
	}

	tabprot <- object@protval

	if (is.null(proteinList)) {
		proteinList <- mcq.get.protlist(object)
	}

	if (is.null(pdf)){
		par(ask=TRUE)
	}
	else{
		pdf(pdf, title=gsub('.pdf$','',pdf))
	}

	tabprot <- merge(object@protval, object@metadata@metadata, by.x="msrun", by.y="row.names")

	for (protein_code in names(proteinList@proteins)) {
		tabcour=tabprot[tabprot$protein==protein_code,]
		protein_name <- object@proteins[protein_code, "accession"]
		if (is.na(protein_name)) {
			protein_name <- protein_code
		}
		title <- paste(protein_name, " (", factorName, ")", sep="")
		visuprot(protein_code,tabcour,tabcour$q,tabcour[,factorName], title=title)
	}



	if (!is.null(pdf)){
		dev.off()
	}

}

