library('MassChroqR')

setwd ('/tmp')

if (file.exists("protpep.RData")) {
	load("protpep.RData")
} else {
	protPep <- read.mcqProtPep('/gorgone/pappso/formation/Decembre2014/XIC_result_prot.tsv', '/gorgone/pappso/formation/Decembre2014/XIC_result_pep.tsv')

	# import des métadata
	metadata <- read.mcq.metadata.ods('/gorgone/pappso/formation/Decembre2014/metadata.ods')

	# Ajout des métadata à l'objet protPep
	protPep <- mcq.merge.metadata(protPep, metadata)
	save(protPep, file="protpep.RData")
}

if (file.exists("protq.RData")) {
	load("protq.RData")
} else {
	qprot <- mcq.protquantity(protPep)
	save(qprot, file="protq.RData")
}

#mcq.get.protlist(qprot)


plist = mcq.protlist(c("GRMZM2G139643_P01 NP_001131946 hypothetical protein LOC100193339 seq=translation; coord=4:229898346..229903473:1; parent_transcript=GRMZM2G139643_T01; parent_gene=GRMZM2G139643",
"GRMZM2G140116_P01 NP_001131498 hypothetical protein LOC100192835 seq=translation; coord=2:197230836..197232295:1; parent_transcript=GRMZM2G140116_T01; parent_gene=GRMZM2G140116"
		, "GRMZM2G004111_P01 B6TW91 Putative uncharacterized protein seq=translation; coord=8:169891088..169891851:-1; parent_transcript=GRMZM2G004111_T01; parent_gene=GRMZM2G004111"
,
"GRMZM2G001898_P01 NP_001142146 hypothetical protein LOC100274311 seq=translation; coord=7:13844415..13850805:1; parent_transcript=GRMZM2G001898_T01; parent_gene=GRMZM2G001898"
))

cat ('mcq.plot.proteins(qprot, factor="extraction", protlist=plist)\n')
mcq.plot.proteins(qprot, factor="extraction", protlist=plist)


cat ("*********************************************\n")
cat ("mcq.plot.proteins test is OK for qprot\n")
cat ("*********************************************\n")

#test plot proteins on peak counting and kruskal


if (file.exists("pkcount.RData")) {
	load("pkcount.RData")
} else {
	pkcount <- mcq.peakcounting(protPep)
	save(pkcount, file="pkcount.RData")
}


if (file.exists("pkcountmindiff.RData")) {
	load("pkcountmindiff.RData")
} else {
	pkcount.mindiff <- mcq.select.mindiff(pkcount, 2, factor='extraction')
	save(pkcount.mindiff, file="pkcountmindiff.RData")
}


if (file.exists("pkcountkruskal.RData")) {
	load("pkcountkruskal.RData")
} else {
	pkcount.kruskal <- mcq.peakcounting.kruskal(pkcount.mindiff, 'extraction')
	save(pkcount.kruskal, file="pkcountkruskal.RData")
}


if (file.exists("pkcountkruskalsignif.RData")) {
	load("pkcountkruskalsignif.RData")
} else {
	pkcount.kruskal.signif <- mcq.select.kruskal.signif(pkcount.kruskal, pvalue=0.01)
	save(pkcount.kruskal.signif, file="pkcountkruskalsignif.RData")
}


mcq.plot.proteins(pkcount, pdf='plotprot_pk.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(pkcount.kruskal, pdf='plotprot_pk_kruskal.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(pkcount.kruskal.signif, pdf='plotprot_pk_kruskal_signif.pdf', protlist = plist)

mcq.plot.proteins(pkcount, pdf='plotprot_pk.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(pkcount.kruskal, pdf='plotprot_pk_kruskal.pdf', factor="extraction")
mcq.plot.proteins(pkcount.kruskal.signif, pdf='plotprot_pk_kruskal_signif.pdf')

cat ("*********************************************\n")
cat ("mcq.plot.proteins test is OK for peak counting\n")
cat ("*********************************************\n")


if (file.exists("spc.RData")) {
	load("spc.RData")
} else {
	spc <- read.mcq.spectralcounting('/gorgone/pappso/formation/Decembre2014/comparspectra.csv')

	# import des métadata
	metadata <- read.mcq.metadata.ods('/gorgone/pappso/formation/Decembre2014/metadata.ods')

	# Ajout des métadata à l'objet protPep
	spc <- mcq.merge.metadata(spc, metadata)
	save(spc, file="spc.RData")
}

#mcq.plot.proteins(spc, pdf='plotprot_spc_all.pdf', factor="extraction")

if (file.exists("spcmindiff.RData")) {
	load("spcmindiff.RData")
} else {
	spcmindiff <- mcq.select.mindiff(spc, 2, factor='extraction')
	save(spcmindiff, file="spcmindiff.RData")
}


if (file.exists("spckruskal.RData")) {
	load("spckruskal.RData")
} else {
	spckruskal <- mcq.spectralcounting.kruskal(spcmindiff, 'extraction')
	save(spckruskal, file="spckruskal.RData")
}


if (file.exists("spckruskalsignif.RData")) {
	load("spckruskalsignif.RData")
} else {
	spckruskalsignif <- mcq.select.kruskal.signif(spckruskal, pvalue=0.01)
	save(spckruskalsignif, file="spckruskalsignif.RData")
}


mcq.plot.proteins(spc, pdf='plotprot_spc.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(spckruskal, pdf='plotprot_spc_kruskal.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(spckruskalsignif, pdf='plotprot_spc_kruskal_signif.pdf', protlist = plist)

mcq.plot.proteins(spc, pdf='plotprot_spc.pdf', factor="extraction", protlist = plist)
mcq.plot.proteins(spckruskal, pdf='plotprot_spc_kruskal.pdf', factor="extraction")
mcq.plot.proteins(spckruskalsignif, pdf='plotprot_spc_kruskal_signif.pdf')

cat ("*********************************************\n")
cat ("mcq.plot.proteins test is OK for spectral counting\n")
cat ("*********************************************\n")



